#include <stdio.h>
#include <stdlib.h>

#define TEXT_FILE "haffman.txt"

/* Специальный узел (который удаляясь, забирает всех своих потомков с собой) */

class Node {
public:
	Node(int v = 0, Node *l = NULL, Node *r = NULL) {
		val = v;
		left = l;
		right = r;
		bukva = '-';
	}
	~Node() {
		if (left) delete left; // Удалаем левого потомка, если он есть
		if (right) delete right; // Удалаем правого потомка, если он есть
	}
	int val;
	char bukva;
	Node *left, *right;
};

/* Функция заключительного этапа алгоритма (оцифровка ребер) */

void edgeNumerator(const Node *node, int n)
{
	static char str[111] = { 0 };

	if (!node->left && !node->right) {
		printf("%c\t|%5i |\t%s\n", node->bukva, node->val, str);
		return;
	}
	
	str[n] = '0'; str[n+1] = '\0';
	if (node->left) edgeNumerator(node->left, n+1);

	str[n] = '1'; str[n+1] = '\0';
	if (node->right) edgeNumerator(node->right, n+1);
}

/* Метод сравнения узлов (по значению его содержимого) */

int myNodeCmp(const void *aa, const void *bb)
{
	Node *a = *((Node **)aa);
	Node *b = *((Node **)bb);
	return (a->val < b->val ? 1 : (a->val > b->val ? -1 : 0));
}

/* Cам алгоритм */

int solveProblem(FILE *fp)
{
	// Если файл вдруг кончился, то конец (вернем неудачу в попытке решения задачи из файла)
	if (feof(fp))
		return 0;
	
	// Читаем значения из файла (если ошибка чтения, то неудача)
	int n = 0;
	if (fscanf(fp, "%i", &n) != 1)
		return 0;
	
	// Чтобы удобно было ограничим n 26 буквами английскими буквами
	if (n > 26) n = 26;
	
	Node **mas = new Node *[n];
	for (int i = 0; i < n; i++) {
		int val;
		if (fscanf(fp, "%i", &val) != 1)
			return 0;
		mas[i] = new Node(val);
		mas[i]->bukva = 'a'+i;
	}
	
	// Находим решение
	int count = n;
	while (count > 1) {
		// Сортируем деревья по значению их корней (по убыванию)
		qsort(mas, count, sizeof(Node *), myNodeCmp);
		// Создаем из последних двух узлов дерево (в корне сумма значений узлов)
		mas[count-2] = new Node(
			mas[count-2]->val + mas[count-1]->val,
			mas[count-2],
			mas[count-1]
		);
		// Число обрабатываемых узлов уменьшилось
		count--;
	}
	
	// Выводим ответ
	edgeNumerator(mas[0], 0);
	printf("\n");

	// Удаляем дерево (mas[0] - его корень), деструктор удалит все что висит на этом дереве
	delete mas[0];

	// Удаляем массив
	delete [] mas;

	// Задача решена
	return 1;
}

/* Старт программы тут */

int main () {
	FILE *fp = fopen(TEXT_FILE, "rt");
	if (!fp) {
		fprintf(stderr, "Can't open file '%s'!\n", TEXT_FILE);
		return -1;
	}
	
	// Решаем все че есть в файле
	while (1) {
		if (!solveProblem(fp))
			break;
	}
	
	fclose(fp);
	
    printf("Konez!\n");
    return 0;
}
